@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -21,9 +21,9 @@
 
 package com.openexchange.authentication.ldap;
 
-import java.io.IOException;
 import java.util.Properties;
 import java.util.concurrent.atomic.AtomicReference;
+import java.util.regex.Pattern;
 import javax.naming.AuthenticationException;
 import javax.naming.Context;
 import javax.naming.InvalidNameException;
@@ -41,7 +41,6 @@ import com.openexchange.authentication.ContextAndUserInfo;
 import com.openexchange.authentication.LoginExceptionCodes;
 import com.openexchange.authentication.LoginInfo;
 import com.openexchange.config.ConfigurationService;
-import com.openexchange.config.ConfigurationServices;
 import com.openexchange.config.DefaultInterests;
 import com.openexchange.config.Interests;
 import com.openexchange.config.Reloadable;
@@ -53,6 +52,7 @@ import com.openexchange.server.ServiceLookup;
 
 /**
  * This class implements the login by using an LDAP for authentication.
+ *
  * @author <a href="mailto:marcus@open-xchange.org">Marcus Klein</a>
  */
 public class LDAPAuthentication implements AuthenticationService, Reloadable {
@@ -148,11 +148,16 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
         throw LoginExceptionCodes.NOT_SUPPORTED.create(LDAPAuthentication.class.getName());
     }
 
+    private static final Pattern PATTERN_USER_NAME = Pattern.compile("[a-zA-Z_0-9-\\s]*");
+
+    private static final Pattern PATTERN_PASSWORD = Pattern.compile("[\\w]*");
+
     /**
      * Tries to bind.
-     * @param uid login name.
-     * @param password password.
-     * @throws LoginException if some problem occurs.
+     *
+     * @param uid The login name.
+     * @param password The password.
+     * @throws LoginException If some problem occurs.
      */
     private String bind(String uid, String password, Properties props, Config config) throws OXException {
         String proxyAs = null;
@@ -172,6 +177,12 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
             }
         }
 
+        // TODO: Use better regex
+        if (false && (!PATTERN_USER_NAME.matcher(uidToUse).matches() || !PATTERN_PASSWORD.matcher(password).matches())) {
+            LOG.error("Missing user mapping for user \"{}\". Login failed.", uidToUse);
+            throw LoginExceptionCodes.INVALID_CREDENTIALS.create();
+        }
+
         LdapContext context = null;
         String dn = null;
         try {
@@ -188,7 +199,7 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
                     }
                     dn = uidToUse;
                 } else {
-                    dn = uidAttribute + '=' + uidToUse + ',' + baseDN;
+                    dn = uidAttribute + '=' + escapeDN(uidToUse) + ',' + baseDN;
                 }
             } else {
                 // get user dn from user
@@ -203,8 +214,8 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
                     aprops.put(Context.SECURITY_AUTHENTICATION, "none");
                 }
                 context = new InitialLdapContext(aprops, null);
-                final String filter = "(&" + config.searchFilter + "(" + uidAttribute + "=" + uidToUse + "))";
-                LOG.debug("Using filter={}", filter);
+                final String filter = "(&" + config.searchFilter + "(" + uidAttribute + "={0}))";
+                LOG.debug("Using filter=(&{}({}={}))", config.searchFilter, uidAttribute, uidToUse);
                 LOG.debug("BaseDN      ={}", baseDN);
                 SearchControls cons = new SearchControls();
                 cons.setSearchScope(config.searchScope);
@@ -212,17 +223,15 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
                 cons.setReturningAttributes(new String[] { "dn" });
                 NamingEnumeration<SearchResult> res = null;
                 try {
-                    res = context.search(baseDN, filter, cons);
+                    res = context.search(baseDN, filter, new Object[] { uidToUse }, cons);
                     if (!res.hasMoreElements()) {
-                        final String errortext = "No user found with " + uidAttribute + "=" + uidToUse;
-                        LOG.error(errortext);
-                        throw LoginExceptionCodes.INVALID_CREDENTIALS_MISSING_USER_MAPPING.create(uidToUse);
+                        LOG.error("No user found with {}={}", uidAttribute, uidToUse);
+                        throw LoginExceptionCodes.INVALID_CREDENTIALS.create();
                     }
 
                     dn = res.nextElement().getNameInNamespace();
                     if (res.hasMoreElements()) {
-                        final String errortext = "Found more than one user with " + uidAttribute + "=" + uidToUse;
-                        LOG.error(errortext);
+                        LOG.error("Found more than one user with {}={}", uidAttribute, uidToUse);
                         throw LoginExceptionCodes.INVALID_CREDENTIALS.create();
                     }
                 } finally {
@@ -254,17 +263,17 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
                 try {
                     if (null == samAccountName) {
                         if (proxyAs != null) {
-                            search = context.search(baseDN, "(displayName=" + uidToUse + ")", searchControls);
-                            searchProxy = context.search(baseDN, "(displayName=" + proxyAs + ")", searchControls);
+                            search = context.search(baseDN, "(displayName={0})", new Object[] { uidToUse }, searchControls);
+                            searchProxy = context.search(baseDN, "(displayName={0})", new Object[] { proxyAs }, searchControls);
                         } else {
-                            search = context.search(baseDN, "(displayName=" + uidToUse + ")", searchControls);
+                            search = context.search(baseDN, "(displayName={0})", new Object[] { uidToUse }, searchControls);
                         }
                     } else {
-                        search = context.search(baseDN, "(sAMAccountName=" + samAccountName + ")", searchControls);
+                        search = context.search(baseDN, "(sAMAccountName={0})", new Object[] { samAccountName }, searchControls);
                     }
                     if (null == search || !search.hasMoreElements()) {
                         LOG.error("No user with displayname {} found.", uidToUse);
-                        throw LoginExceptionCodes.INVALID_CREDENTIALS_MISSING_USER_MAPPING.create(uidToUse);
+                        throw LoginExceptionCodes.INVALID_CREDENTIALS.create(uidToUse);
                     }
 
                     SearchResult next = search.next();
@@ -294,7 +303,7 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
             if (context != null) {
                 try {
                     context.close();
-                } catch (NamingException e) {
+                } catch (Exception e) {
                     LOG.error("", e);
                 }
             }
@@ -302,7 +311,7 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
     }
 
     /**
-     * Initializes the properties and configuration for the ldap authentication.
+     * Initializes the properties and configuration for the LDAP authentication.
      *
      * @return The properties for the JNDI context as well as the configuration for LDAP authentication to use
      * @throws LoginException If configuration fails.
@@ -400,7 +409,7 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
     /**
      * Splits user name and context.
      *
-     * @param loginInfo combined information separated by an <code>"@"</code> sign.
+     * @param loginInfo The combined information separated by an <code>'@'</code> character.
      * @return The context and user name.
      * @throws LoginException if no separator is found.
      */
@@ -412,9 +421,9 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
     @Override
     public void reloadConfiguration(ConfigurationService configService) {
         try {
-            Properties properties = ConfigurationServices.loadPropertiesFrom(configService.getFileByName("ldapauth.properties"));
+            Properties properties = configService.getFile("ldapauth.properties");
             propsAndConfigReference.set(init(properties));
-        } catch (OXException | IOException e) {
+        } catch (Exception e) {
             LOG.error("Error reloading configuration for bundle com.openexchange.authentication.ldap: {}", e);
         }
     }
@@ -439,4 +448,89 @@ public class LDAPAuthentication implements AuthenticationService, Reloadable {
         }
     }
 
+    // -------------------------------------------------------------------------------------------------------------------------------------
+
+    /**
+     * Escapes data for use in an LDAP distinguished name.
+     *
+     * @param name The data to escape
+     * @return The escaped data
+     */
+    private static String escapeDN(String name) {
+        int length = name.length();
+
+        StringBuilder sb = null;
+        if ((length > 0) && ((name.charAt(0) == ' ') || (name.charAt(0) == '#'))) {
+            sb = new StringBuilder(length);
+            sb.append('\\'); // add the leading backslash if needed
+        }
+
+        for (int i = 0; i < length; i++) {
+            char curChar = name.charAt(i);
+            switch (curChar) {
+                case '\\':
+                    if (sb == null) {
+                        sb = initStringBuilder(i, name, length);
+                    }
+                    sb.append("\\\\");
+                    break;
+                case ',':
+                    if (sb == null) {
+                        sb = initStringBuilder(i, name, length);
+                    }
+                    sb.append("\\,");
+                    break;
+                case '+':
+                    if (sb == null) {
+                        sb = initStringBuilder(i, name, length);
+                    }
+                    sb.append("\\+");
+                    break;
+                case '"':
+                    if (sb == null) {
+                        sb = initStringBuilder(i, name, length);
+                    }
+                    sb.append("\\\"");
+                    break;
+                case '<':
+                    if (sb == null) {
+                        sb = initStringBuilder(i, name, length);
+                    }
+                    sb.append("\\<");
+                    break;
+                case '>':
+                    if (sb == null) {
+                        sb = initStringBuilder(i, name, length);
+                    }
+                    sb.append("\\>");
+                    break;
+                case ';':
+                    if (sb == null) {
+                        sb = initStringBuilder(i, name, length);
+                    }
+                    sb.append("\\;");
+                    break;
+                default:
+                    if (sb != null) {
+                        sb.append(curChar);
+                    }
+            }
+        }
+        if ((length > 1) && (name.charAt(length - 1) == ' ')) {
+            if (sb == null) {
+                sb = initStringBuilder(length - 1, name, length);
+            }
+            sb.insert(sb.length() - 1, '\\'); // add the trailing backslash if needed
+        }
+        return sb == null ? name : sb.toString();
+    }
+
+    private static StringBuilder initStringBuilder(int index, String source, int length) {
+        StringBuilder s = new StringBuilder(length);
+        if (index > 0) {
+            s.append(source, 0, index);
+        }
+        return s;
+    }
+
 }
