@@ -1,5 +1,5 @@
 /*
- * @copyright Copyright (c) OX Software GmbH, Germany <info@open-xchange.com>
+ * @copyright Copyright (c) Open-Xchange GmbH, Germany <info@open-xchange.com>
  * @license AGPL-3.0
  *
  * This code is free software: you can redistribute it and/or modify
@@ -25,7 +25,6 @@ import java.util.Properties;
 import com.openexchange.authentication.AuthenticationService;
 import com.openexchange.authentication.ldap.LDAPAuthentication;
 import com.openexchange.config.ConfigurationService;
-import com.openexchange.config.ConfigurationServices;
 import com.openexchange.config.Reloadable;
 import com.openexchange.net.ssl.SSLSocketFactoryProvider;
 import com.openexchange.osgi.HousekeepingActivator;
@@ -53,7 +52,7 @@ public class AuthLDAPActivator extends HousekeepingActivator {
 
     /**
      * {@inheritDoc}
-     * 
+     *
      * @throws Exception if the authentication class can not be initialized.
      */
     @Override
@@ -61,7 +60,7 @@ public class AuthLDAPActivator extends HousekeepingActivator {
         LOG.info("Starting ldap authentication service.");
 
         final ConfigurationService config = getService(ConfigurationService.class);
-        final Properties props = ConfigurationServices.loadPropertiesFrom(config.getFileByName("ldapauth.properties"));
+        final Properties props = config.getFile("ldapauth.properties");
 
         LDAPAuthentication impl = new LDAPAuthentication(props, this);
         registerService(AuthenticationService.class, impl, null);
